/*
 *  Copyright (c) 1999 Sun Microsystems, Inc., 901 San Antonio Road,
 *  Palo Alto, CA 94303, U.S.A.  All Rights Reserved.
 *
 *  Sun Microsystems, Inc. has intellectual property rights relating
 *  to the technology embodied in this software.  In particular, and
 *  without limitation, these intellectual property rights may include
 *  one or more U.S. patents, foreign patents, or pending
 *  applications.  Sun, Sun Microsystems, the Sun logo, Java, KJava,
 *  and all Sun-based and Java-based marks are trademarks or
 *  registered trademarks of Sun Microsystems, Inc.  in the United
 *  States and other countries.
 *
 *  This software is distributed under licenses restricting its use,
 *  copying, distribution, and decompilation.  No part of this
 *  software may be reproduced in any form by any means without prior
 *  written authorization of Sun and its licensors, if any.
 *
 *  FEDERAL ACQUISITIONS:  Commercial Software -- Government Users
 *  Subject to Standard License Terms and Conditions
 */

/*
 * WARNING - THIS IS AN EXPERIMENTAL FEATURE OF KVM THAT MAY, OR MAY NOT
 * EXIST IN A FUTURE VERSION OF THIS PRODUCT. IT IS NOT A PART OF THE
 * CLDC SPECIFICATION AND IS PROVIDED FOR ILLUSTRATIVE PURPOSES ONLY
 */

/*
 * NOTE - This code is written a somewhat unusual, and not very object oriented, way.
 * The principal reason for this is that we intend to translate much of this
 * protocol into C and place it in the KVM kernel where is will be much smaller
 * and faster.
 */

package com.sun.cldc.io.palm.storage;
import java.io.*;
import javax.microedition.io.*;
import com.sun.cldc.io.connections.*;
import com.sun.kjava.*;

/**
 * This implements the default "storage:" protocol for Palm
 * <p>
 * This is a REFERENCE IMPLEMENTATION for the Palm. The object of this
 * implementation is correctness not efficiency, and in any case it uses the
 * com.sun.kjava.Database class whose life expediency is not long.
 *
 * @author  Nik Shaylor
 * @version 2.0 3/1/2000
 */


public class Protocol extends ProtocolStub {

    final static int FIRSTRECORD = 0;
    final static int LASTRECORD  = 32767;

    /* Database handle */
    private Database db;

    /* Current record */
    private byte[] buffer;

    /* Current record state */
    private boolean modified;

    /* Current byte position */
    private int position;

    /* Length of live data in the current buffer */
    private int count;

    /*
     * isDefined
     */
    boolean isDefined() {
        if(selection == NOSELECTION) {
            throw new RuntimeException("isDefined called without selection");
        }
        if(buffer == null) {
            modified = false;
            if(selection < db.getNumberOfRecords()) {
                 buffer = db.getRecord(selection);
            }
            if(buffer != null) {
                count = buffer.length;
            } else {
                count = -1;
            }
        }
        return buffer != null;
    }

    /*
     * defineRecord
     */
    void defineRecord() {
        if(!isDefined()) {
            buffer = new byte[0];
            count = 0;
            modified = true;
        }
    }

    /*
     * prim_initialise
     */
    public Connection prim_openProtocol(String openName, String parms, int mode) throws IOException {

System.out.println("prim_openProtocol "+openName+" parms "+parms);

        try {
            parse(openName);
            if(type == 0) {
                OuterProtocol c = new OuterProtocol();
System.out.println("prim_openProtocol calling outer "+openName+parms);
                c.openPrim(openName+parms, mode, false);
                return c;
            }
            int dbmode = 0;
            if((mode&Connector.READ) != 0) {
                dbmode |= Database.READONLY;
            }
            if((mode&Connector.WRITE) != 0) {
                dbmode |= Database.READWRITE;
            }
System.out.println("prim_openProtocol db open "+creator+" "+type+" "+dbmode);
            db = new Database(type, creator, dbmode);
            if(db != null && !db.isOpen()) {
System.out.println("prim_openProtocol db !open "+creator+" "+type+" "+dbmode);
                db.close();
                db = null;
            }
        } catch(IOException x) {
System.out.println("prim_openProtocol IOException "+x);
            db = null;
        }
        if(db == null) {
System.out.println("prim_openProtocol cnf "+creator+" "+type+" "+mode);
            throw new ConnectionNotFoundException();
        }
        return this;
    }

    /*
     * prim_closeProtocol
     */
    public void prim_closeProtocol() throws IOException {
        prim_clearSelection();
        db.close();
        db        = null;
        creator   = NOCREATOR;
        type      = NOTYPE;
        selection = NOSELECTION;
    }

    /*
     * prim_countItems -- TEMP
     */
    public int prim_countItems() {
        flushBuffer();
        return db.getNumberOfRecords();
    }

    /*
     * setSelection
     */
    private void setSelection(int record) throws IOException {
        flushBuffer();
        modified  = false;
        buffer    = null;
        selection = record;
    }

    /*
     * prim_clearSelection
     */
    public void prim_clearSelection() throws IOException {
        setSelection(NOSELECTION);
    }

    /*
     * prim_isSelected
     */
    public boolean prim_isSelected() {
        return selection != NOSELECTION;
    }

    /*
     * getSelectionName
     */
    public String prim_getSelectionName() throws IOException {
        return String.valueOf(prim_getSelectionNumber());
    }

    /*
     * getSelectionNumber
     */
    public int prim_getSelectionNumber() throws IOException {
        return selection;
    }

    /*
     * prim_realOpen
     */
    public void prim_realOpen() throws IOException {
    }

    /*
     * prim_exists
     */
    public boolean prim_exists() {
        return isDefined();
    }

    /*
     * prim_isDirectory
     */
    public boolean prim_isDirectory() {
        return false;
    }

    /*
     * prim_lengthOf
     */
    public long prim_lengthOf() throws IOException {
        if(isDefined()) {
            return count;
        } else {
            return -1;
        }
    }

    /*
     * prim_setLength
     */
    public void prim_setLength(long val) throws IOException {
        if(!isDefined()) {
            throw new RuntimeException("nfd in prim_setLength");
        }
        int len = (int)val;
        if(len == count) {
            return;
        }
        if(len < count) {
            count = len;
            if(position > count) {
                position = count;
            }
        } else {
            int save = position;
            prim_seek(len-1);
            prim_write(0);
            position = save;
        }
    }

    /*
     * prim_timeOf
     */
    public long prim_timeOf() throws IOException {
        if(!isDefined()) {
            throw new RuntimeException("nfd in prim_timeOf");
        }
        return -1;
    }

    /*
     * findItemFrom -- TEMP
     */
/*
    private int findItemFrom(int i) throws IOException {
        for(; i <= LASTRECORD ; i++) {
            setSelection(i);
            if(prim_exists()) {
                return i;
            }
        }
        return NOSELECTION;
    }
*/

    /*
     * prim_findFirstItemAndSelect
     */
    public void prim_findFirstItemAndSelect() throws IOException {
        //findItemFrom(FIRSTRECORD);
        setSelection(0);
    }

    /*
     * prim_findItemAndSelectByInt
     */
    public void prim_findItemAndSelectByInt(int i) throws IOException {
        setSelection(i);
    }

    /*
     * prim_findItemAndSelect
     */
    public void prim_findItemAndSelect(String name) throws IOException {
        prim_findItemAndSelectByInt(Integer.parseInt(name));
    }

    /*
     * prim_findItemAfterSelectionAndSelect
     */
    public void prim_findItemAfterSelectionAndSelect() throws IOException {
        setSelection(selection+1);
    }

    /*
     * prim_createAndSelect -- TEMP
     */
    public void prim_createAndSelect() throws IOException {
/*
        int last = FIRSTRECORD-1;
        for(int i = FIRSTRECORD ; i <= LASTRECORD ; i++) {
            setSelection(i);
            if(prim_exists()) {
                last = i;
            }
        }
*/
        int num = db.getNumberOfRecords();
        prim_createFileAndSelectByInt(num);
    }

    /*
     * prim_createFileAndSelectByInt
     */
    public void prim_createFileAndSelectByInt(int i) throws IOException {
        if(i == LASTRECORD) {
            throw new IOException("Database full");
        }
        setSelection(i);
        if(prim_exists()) {
            throw new IOException("Record exists "+i);
        }
    }

    /*
     * prim_createFileAndSelect
     */
    public void prim_createFileAndSelect(String name) throws IOException {
        prim_createFileAndSelectByInt(Integer.parseInt(name));
    }

    /*
     * prim_deleteItem
     */
    public boolean prim_deleteItem() throws IOException {
        int i = selection;
        clearSelection();
        db.deleteRecord(i);
        setSelection(i);
        return true;
    }

    /*
     * prim_renameFileByInt
     */
    public boolean prim_renameFileByInt(int i) throws IOException {
        throw new RuntimeException("prim_renameFileByInt not defined");
/**
        if(!isDefined()) {
            throw new RuntimeException("nfd in prim_renameFileByInt");
        }
        if(db.getRecord(i) != null) {
            throw new IOException("New Record exists "+i);
        }
        db.setRecord(i, buffer);
        prim_deleteItem();
        return true ;
**/
    }

    /*
     * prim_renameFile
     */
    public boolean prim_renameFile(String name2) throws IOException {
        return prim_renameFileByInt(Integer.parseInt(name2));
    }

    /*
     * prim_createDirectoryAndSelect
     */
    public void prim_createDirectoryAndSelect(String name) throws IOException {
        throw new ndf();
    }

    /*
     * prim_renameDirectory
     */
    public boolean prim_renameDirectory(String name2) throws IOException {
        throw new ndf();
    }

    /*
     * prim_canRead
     */
    public boolean prim_canRead() throws IOException {
        return true;
    }

    /*
     * prim_canWrite
     */
    public boolean prim_canWrite() throws IOException {
        return true;
    }

    /*
     * prim_setReadable
     */
    public boolean prim_setReadable(boolean tf) throws IOException {
        return tf == true;
    }

    /*
     * prim_setWritable
     */
    public boolean prim_setWritable(boolean tf) throws IOException {
        return tf == true;
    }

    /*
     * prim_seek
     */
    public void prim_seek(long pos) throws IOException {
        position = (int)pos;
    }

    /*
     * prim_getPosition
     */
    public long prim_getPosition() throws IOException {
        return position;
    }

    /*
     * prim_read
     */
    public int prim_read() throws IOException {
        if(!isDefined() || position == count) {
            return -1;
        }
        return buffer[position++];
    }

    /*
     * prim_readBytes
     */
    public int prim_readBytes(byte b[], int off, int len) throws IOException {
/*
        if(position == 0 && state != MODIFIED) {
            buffer = null;
            position = readIntoBuffer(selection, b, off, len);
            return position;
        }
*/
        if(!isDefined() || position == count) {
            return 0;
        }
        int l = count - position;
        if(l > len) {
            l = len;
        }
        System.arraycopy(buffer, position, b, off, len);
        position += l;
        return l;
    }

    /*
     * prim_write
     */
    public void prim_write(int b) throws IOException {
        defineRecord();
        if(position == buffer.length) {
            growBuffer(buffer.length+1);
        }
        buffer[position++] = (byte)b;
        if(count < position) {
            count = position;
        }
        modified = true;
    }

    /*
     * prim_writeBytes
     */
    public void prim_writeBytes(byte b[], int off, int len) throws IOException {
        if(off == 0 && (buffer == null || (position == 0 && len == count))) {
            db.setRecord(selection, b);
            buffer = null;
            return;
        }
        defineRecord();
        if((position+len) >= buffer.length) {
            growBuffer(position+len);
        }
        System.arraycopy(b, off, buffer, position, len);
        position += len;
        if(count < position) {
            count = position;
        }
        modified = true;
    }

    /*
     * growBuffer
     */
    private void growBuffer(int len) {
        int newcount = buffer.length + len;
        byte newbuf[] = new byte[Math.max(buffer.length * 2, newcount)];
        System.arraycopy(buffer, 0, newbuf, 0, buffer.length);
        buffer = newbuf;

    }

    /*
     * flushBuffer
     */
    private void flushBuffer() {
        if(buffer != null && modified) {
            if(count == buffer.length) {
                db.setRecord(selection, buffer);
            } else {
                byte[] b = new byte[count];
                System.arraycopy(buffer, 0, b, 0, count);
                db.setRecord(selection, buffer);
            }
        }
    }

}
