/*
 *  Copyright (c) 1999 Sun Microsystems, Inc., 901 San Antonio Road,
 *  Palo Alto, CA 94303, U.S.A.  All Rights Reserved.
 *
 *  Sun Microsystems, Inc. has intellectual property rights relating
 *  to the technology embodied in this software.  In particular, and
 *  without limitation, these intellectual property rights may include
 *  one or more U.S. patents, foreign patents, or pending
 *  applications.  Sun, Sun Microsystems, the Sun logo, Java, KJava,
 *  and all Sun-based and Java-based marks are trademarks or
 *  registered trademarks of Sun Microsystems, Inc.  in the United
 *  States and other countries.
 *
 *  This software is distributed under licenses restricting its use,
 *  copying, distribution, and decompilation.  No part of this
 *  software may be reproduced in any form by any means without prior
 *  written authorization of Sun and its licensors, if any.
 *
 *  FEDERAL ACQUISITIONS:  Commercial Software -- Government Users
 *  Subject to Standard License Terms and Conditions
 */

/*
 * WARNING - THIS IS AN EXPERIMENTAL FEATURE OF KVM THAT MAY, OR MAY NOT
 * EXIST IN A FUTURE VERSION OF THIS PRODUCT. IT IS NOT A PART OF THE
 * CLDC SPECIFICATION AND IS PROVIDED FOR ILLUSTRATIVE PURPOSES ONLY
 */

/*
 * NOTE - This code is written a somewhat unusual, and not very object oriented, way.
 * The principal reason for this is that we intend to translate much of this
 * protocol into C and place it in the KVM kernel where is will be much smaller
 * and faster.
 */

package com.sun.cldc.io.palm.storage;
import java.io.*;
import javax.microedition.io.*;
//import com.sun.kjava.Database;

/**
 * This implements the default "storage:" protocol for Palm
 * <p>
 * This is a REFERENCE IMPLEMENTATION for the Palm. The object of this
 * implementation is correctness not efficiency, and in any case it uses the
 * com.sun.kjava.Database class whose life expediency is not long.
 *
 * @author  Nik Shaylor
 * @version 2.0 3/1/2000
 */


abstract class ProtocolStub extends com.sun.cldc.io.j2me.storage.ProtocolNative {

    static final int NOCREATOR   = -1;
    static final int NOSELECTION = -1;
    static final int NOTYPE      = -1;

    static final int DEFAULT_CREATOR = (((byte)'k') << 24) |
                                       (((byte)'J') << 16) |
                                       (((byte)'a') << 8)  |
                                       (((byte)'x') << 0)  ;


    /* Current Palm database 'creator' */
    public int creator = NOCREATOR;

    /* Current Palm database 'type' */
    public int type = NOTYPE;

    /* Current Palm database type of database record number' */
    public int selection = NOSELECTION;

    /*
     * prim_initialise
     */
    abstract public Connection prim_openProtocol(String openName, String parms, int mode) throws IOException;

    public long prim_availableSpace() throws IOException {
        throw new ndf(0);
    }

    /*
     * Functions to be overridden.
     */
    public void    prim_closeProtocol() throws IOException                                             { throw new ndf(1); }
    public void    prim_realOpen() throws IOException                                                  { throw new ndf(2); }
    public void    prim_clearSelection() throws IOException                                            { throw new ndf(3); }
    public String  prim_getSelectionName() throws IOException                                          { throw new ndf(4); }
    public int     prim_getSelectionNumber() throws IOException                                        { throw new ndf(5); }
    public boolean prim_isSelected() throws IOException                                                { throw new ndf(6); }
    public void    prim_findFirstItemAndSelect() throws IOException                                    { throw new ndf(7); }
    public void    prim_findItemAndSelect(String name) throws IOException                              { throw new ndf(8); }
    public void    prim_findItemAndSelectByInt(int i) throws IOException                               { throw new ndf(9); }
    public void    prim_findItemAfterSelectionAndSelect() throws IOException                           { throw new ndf(10); }
    public int     prim_countItems()  throws IOException                                               { throw new ndf(11); }
    public void    prim_createAndSelect() throws IOException                                           { throw new ndf(12); }
    public void    prim_createFileAndSelect(String name) throws IOException                            { throw new ndf(13); }
    public void    prim_createFileAndSelectByInt(int i) throws IOException                             { throw new ndf(14); }
    public void    prim_createDirectoryAndSelect(String name) throws IOException                       { throw new ndf(15); }
    public boolean prim_deleteItem() throws IOException                                                { throw new ndf(16); }
    public boolean prim_renameFile(String name2) throws IOException                                    { throw new ndf(17); }
    public boolean prim_renameFileByInt(int i) throws IOException                                      { throw new ndf(18); }
    public boolean prim_renameDirectory(String name2) throws IOException                               { throw new ndf(19); }
    public long    prim_lengthOf() throws IOException                                                  { throw new ndf(20); }
    public void    prim_setLength(long len) throws IOException                                         { throw new ndf(21); }
    public long    prim_timeOf() throws IOException                                                    { throw new ndf(22); }
    public boolean prim_exists() throws IOException                                                    { throw new ndf(23); }
    public boolean prim_isDirectory() throws IOException                                               { throw new ndf(24); }
    public boolean prim_canRead() throws IOException                                                   { throw new ndf(25); }
    public boolean prim_canWrite() throws IOException                                                  { throw new ndf(26); }
    public boolean prim_setReadable(boolean tf) throws IOException                                     { throw new ndf(27); }
    public boolean prim_setWritable(boolean tf) throws IOException                                     { throw new ndf(28); }
    public void    prim_seek(long pos) throws IOException                                              { throw new ndf(29); }
    public long    prim_getPosition() throws IOException                                               { throw new ndf(30); }
    public int     prim_read() throws IOException                                                      { throw new ndf(31); }
    public int     prim_readBytes(byte b[], int off, int len) throws IOException                       { throw new ndf(32); }
    public void    prim_write(int b) throws IOException                                                { throw new ndf(33); }
    public void    prim_writeBytes(byte b[], int off, int len) throws IOException                      { throw new ndf(34); }


// ---------------------------------------------------------------------------
//                           The open string parser
// ---------------------------------------------------------------------------

/*
 * The following are the set of legal strings that can be used to identify databases and records
 *
 *                      creator   type    record  result
 *                      -------   ----    ------  ------
 *
 * ""                   default   -       -       c--
 * "/"                  default   -       -       c--
 * "/creator"           default   -       -       c--
 * "/creator/"          default   -       -       c--
 * "/creator/c"         c         -       -       c--
 * "/creator/c/"        c         -       -       c--
 * "/creator/c/t"       c         t       -       ct-
 * "/creator/c/t/"      c         t       -       ct-
 * "/creator/c/t/n"     c         t       n       ctn
 *
 * "/type"              default   -       -       c--
 * "/type/"             default   -       -       c--
 * "/type/t"            -         t       -       -t-
 * "/type/t/"           -         t       -       -t-
 * "/type/t/c"          c         t       -       ct-
 * "/type/t/c/"         c         t       -       ct-
 * "/type/t/c/n"        c         t       n       ctn
 *
 * "t/"                 default   t       -       ct-
 * "t/n"                default   t       n       ctn
 *
 * Where c = creator ID (or '+' for the default)
 *       t = type ID
 *       n = record number
 *
 * For the future:
 *
 * "/name"
 * "/name/"
 * "/name/"
 * "/name/n"
 */

     static final int END   = 0;
     static final int SLASH = 1;
     static final int PLUS  = 2;

     String parseString;
     int    ptr = 0;
     boolean Cmode;

     /*
      * getc - get a character
      */
     int getc() {
         if(ptr == parseString.length()) {
             return END;
         }
         int ch = parseString.charAt(ptr++);
         if(ch == '/') {
             return SLASH;
         }
         if(ch == '+') {
             return PLUS;
         }
         return ch;
     }

     /*
      * geti - get a character that can be part of an identifier
      */
     int geti() {
         int ch  = getc();
         if(ch == '/') {
            --ptr;
            return 0;
         }
         if(ch == END) {
            return 0;
         }
         return ch;
     }

     /*
      * getn - get a number but do not advance the pointer
      */
     int getn() {
         int save = ptr;
         int val = 0;
         while(true) {
             int ch = getc();
             if(ch < '0' || ch > '9') {
                 break;
             }
             val = (val * 10) + (ch - '0');
         }
         ptr = save;
         return val;
     }

     /*
      * token
      */
     int token() {
         int tok = getc();
         if(tok != SLASH && tok != END && tok != PLUS) {
             tok = (tok << 24) + (geti() << 16) + (geti() << 8) + geti();
         }

//System.out.println("Token "+tok);
         return tok;
     }

     /*
      * parse
      */
    void parse(String name) throws IOException {
        parseString = name;

//System.out.println("Parsing "+name);
        boolean res;
        ptr         = 0;
        creator     = DEFAULT_CREATOR;
        type        = 0;
        selection   = -1;

        if(parseString.equals("") || parseString.equals("/")) {
           return;
        }

        if(parseString.startsWith("/creator")) {
           ptr = 9;
           Cmode = true;
           res = parse_();
        } else if (parseString.startsWith("/type")) {
           ptr = 6;
           Cmode = false;
           res = parse_();
        } else {
           res = false;
           int val = token();
           if(val != SLASH && val != END && val != PLUS) {                      //      x?
               type = val;
               val = token();
               if(val == SLASH) {                                               //      x/?
                   int num = getn();
                   val = token();
                   if(val == END) {
                       res = true;                                              //      x/
                   } else {
                       if(val != SLASH && val != END && val != PLUS) {          //      x/y?
                           val = token();
                           if(val == END) {                                     //      x/y
                               selection = num;
                               res = true;
                           }
                       }
                   }
               }
           }
       }

//System.out.print(" res="+res);
//System.out.print(" creator="+creator);
//System.out.print(" type="+type);
//System.out.println(" selection="+selection);

       if(!res) {
           throw new IOException("?");
       }
   }

    /*
     * parse_
     */
    boolean parse_() {                          //      /createOrType?
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType
                return true;
            }
            case SLASH: {
                return parse_S();               //      /createOrType/?
            }
            default: {
                return false;
            }
        }
    }

    /*
     * parseC_S
     */
    boolean parse_S() {                         //      /createOrType/?
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType/
                return true;
            }
            case SLASH: {
                return false;                   //      /createOrType//?
            }
            case PLUS: {
                if(!Cmode) {
                    return false;
                }
                return parse_Sx();              //      /createOrType/+?
            }
            default: {
                if(Cmode) {
                    creator = val;
                } else {
                    type = val;
                }
                return parse_Sx();              //      /createOrType/x?
            }
        }
    }

    /*
     * parse_Sx
     */
    boolean parse_Sx() {                        //      /createOrType/x?
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType/x
                return true;
            }
            case SLASH: {
                return parse_SxS();             //      /createOrType/x/?
            }
            default: {
                return false;                   //      /createOrType/x
            }
        }
    }


    /*
     * parseC_SxS
     */
    boolean parse_SxS() {                       //      /createOrType/x/
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType/x/
                return true;
            }
            case SLASH: {
                return false;                   //      /createOrType/x//
            }
            case PLUS: {
                if(Cmode) {
                    return false;               //      /creator/t/+?
                }
                return parse_SxSx();            //      /type/t/+?

            }
            default: {
                if(Cmode) {
                    type = val;
                } else {
                    creator = val;
                }
                return parse_SxSx();            //      /createOrType/x/y?
            }
        }
    }

    /*
     * parseC_SxSx
     */
    boolean parse_SxSx() {                      //      /createOrType/x/y?
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType/x/y
                return true;
            }
            case SLASH: {
                return parse_SxSxS();           //      /createOrType/x/y/
            }
            default: {
                return false;
            }
        }
    }


    /*
     * parseC_SxSxS
     */
    boolean parse_SxSxS() {                     //      /createOrType/c/t/?
        int num = getn();
        int val = token();
        switch(val) {
            case END: {                         //      /createOrType/c/t/
                return true;
            }
            case SLASH: {
                return false;                   //      /createOrType/x/t//
            }
            case PLUS: {
                return false;                   //      /createOrType/x/t/+
            }
            default: {
                if(token() != END) {
                    return false;
                }
                selection = num;
                return true;

            }
        }
    }
}
