/*
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

/*=========================================================================
 * Spotless Virtual Machine
 *=========================================================================
 * SYSTEM:    Spotless VM
 * SUBSYSTEM: Native function interface
 * FILE:      inetlib.c
 * OVERVIEW:  This file defines the native functions needed
 *            by the Java virtual machine. The implementation
 *            is _not_ based on JNI (Java Native Interface),
 *            because it seems too complicated for small devices.
 * AUTHOR:    Bill Pittore SunLabs 12/99
 * NOTE:      Many functions in this file are platform-specific,
 *            and require extra work when porting the system to
 *            run on another machine.
 *=======================================================================*/

/*=========================================================================
 * Include files
 *=======================================================================*/

#include "global.h"
#include <INetMgr.h>
#include <CTP.h>
#include <KVM.h>

/*=======================================================================
 * Globals
 *=======================================================================*/

static Word AppINetRefnum;
static Word AppINetTimeout = 2000;
static Handle AppInetH;
static Err lastErr;


/*=======================================================================
 * Declarations
 *=======================================================================*/

void Java_com_sun_cldc_io_palm_http_Inet_initProto(void);
void Java_com_sun_cldc_io_palm_http_Inet_showSignal(void);
void Java_com_sun_cldc_io_palm_http_Inet_openHTTPsocket(void);
void Java_com_sun_cldc_io_palm_http_Inet_HTTPCreateAndSendReq(void);
void Java_com_sun_cldc_io_palm_http_Inet_readHTTPsocket(void);
void Java_com_sun_cldc_io_palm_http_Inet_closeHTTPsocket(void);
void Java_com_sun_cldc_io_palm_http_Inet_getlasterror(void);
void FinalizeINetLib(void);

/*=========================================================================
 * FUNCTION:      initProto(V)I
 * CLASS:         com/sun/kjava/system/palm/protocol/http/Inet
 * TYPE:          native function
 * OVERVIEW:
 * INTERFACE (operand stack manipulation):
 *   parameters:  this (implicitly)
 *   returns:     inet handle ; -1 error
 *=======================================================================*/

void Java_com_sun_cldc_io_palm_http_Inet_initProto()
{
        Err err;
        Handle inetH;
        Word index;
        DWord conv = ctpConvNone;       // no compression

        lastErr = 0;

        err = SysLibFind("INet.lib", &AppINetRefnum);
        if (err) {
                lastErr = err;
                AppINetRefnum = 0;
				AppInetH = 0;
                pushStack(-1);
                return;
        }
        err = INetLibConfigIndexFromName(AppINetRefnum,
                (INetConfigNamePtr)inetCfgNameCTPDefault, &index);
        if (err) {
                lastErr = err;
                AppINetRefnum = 0;
				AppInetH = 0;
                pushStack(-2);
                return;
        }
		if (AppInetH == 0) {
        	err = INetLibOpen(AppINetRefnum, index, 0, NULL, 0, &inetH);
        	if (err) {
                	lastErr = err;
                	AppINetRefnum = 0;
                	pushStack(-3);
                	return;
        	}
        	err = INetLibSettingSet(AppINetRefnum, inetH,
			    inetSettingConvAlgorithm,
               	&conv, sizeof(conv));
        	if (err) {
                	lastErr = err;
                	INetLibClose(AppINetRefnum, inetH);
                	AppINetRefnum = 0;
                	pushStack(-4);
                	return;
        	}
        	AppInetH = inetH;       // keep around for finalize
		}
        pushStack(int)(AppInetH);
}

void Java_com_sun_cldc_io_palm_http_Inet_showSignal()
{

        int y = popStack();
        int x = popStack();

        // Show signal strength meter
        //INetLibWiCmd (AppINetRefnum, wiCmdInit, 0, 0);
//      INetLibWiCmd (AppINetRefnum, wiCmdSetLocation, x, y);
//      INetLibWiCmd (AppINetRefnum, wiCmdSetEnabled, 1, 0);
		if (AppINetRefnum == 0) {
                raiseException("com/sun/cldc/io/palm/http/InetException");
				return;
		}
        INetLibWiCmd (AppINetRefnum, 0, 140, 1);
        INetLibWiCmd (AppINetRefnum, 5, 76, 1);
        INetLibWiCmd (AppINetRefnum, 2, 1, 1);
        oneLess;
        return;
}

void Java_com_sun_cldc_io_palm_http_Inet_openHTTPsocket()
{
        Err err;
        Handle sockH;
        Handle inetH = (Handle)popStack();
		int scheme = popStack();

        lastErr = 0;

        err = INetLibSockOpen(AppINetRefnum, inetH, scheme, &sockH);
        if (err) {
                lastErr = err;
                topStack = -1;
                return;
        }
        topStack = (int)sockH;
}

void Java_com_sun_cldc_io_palm_http_Inet_HTTPCreateAndSendReq()
{
        INSTANCE referer = (INSTANCE)popStack();
        INSTANCE resource = (INSTANCE)popStack();
        BYTEARRAY data = (BYTEARRAY)popStack();           // Data if POST
                BytePtr ptr = NULL;
        INSTANCE verb = (INSTANCE)popStack();           // probably 'GET'
        unsigned char resstr[256];
        unsigned char verbstr[256];
        Handle sockH = (Handle)popStack();
        Err err;

        lastErr = 0;

        oneLess;
        if (resource != NIL) {
                getStringContentsSafely((STRING_INSTANCE)resource, (char *)&resstr, 256);
        } else {
                StrCopy((char *)resstr, "index.html");
        }

        if (verb != NIL) {
                getStringContentsSafely((STRING_INSTANCE)verb, (char *)&verbstr, 256);
        } else {
                StrCopy((char *)verbstr, "GET");
        }

        err = INetLibSockHTTPReqCreate(AppINetRefnum, sockH, verbstr, resstr, NULL);
        if (err) {
                lastErr = err;
                raiseException("com/sun/cldc/io/palm/http/InetException");
				return;
        }

        if (data != NIL) {
                ptr = (unsigned char *)&data->bdata[0];
        }

        err = INetLibSockHTTPReqSend(AppINetRefnum, sockH,
            (data != NULL) ? ptr : NULL, 0, AppINetTimeout);
        if (err) {
                lastErr = err;
                raiseException("com/sun/cldc/io/palm/http/InetException");
				return;
        }
}

void Java_com_sun_cldc_io_palm_http_Inet_readHTTPsocket()
{
        DWord count =  (DWord)popStack();
        Word offset =  popStack();
        BYTEARRAY buf = (BYTEARRAY)popStack();
        Handle sockH = (Handle)popStack();
        BytePtr ptr = (BytePtr)&buf->bdata[offset];
        DWord nleft = count;
        DWord nread;
        Err err;
        DWord total = 0;

        lastErr = 0;

        do {
                err = INetLibSockRead(AppINetRefnum, sockH, ptr, nleft, &nread,
                        AppINetTimeout);
                if (err) {
                      lastErr = err;
                      topStack = -2;
                      return;
                }
                // just return what we've read so far (may be zero, that's ok)
                // if we got zero back then the connection is closed so return -1
                // so eof gets set
                if (nread == 0) {
                        if (total > 0)
                                topStack = total;
                        else
                                topStack = -1;
                        return;
                }
                total += nread;
                nleft -= nread;
                ptr   += nread;
        } while (nleft);

        topStack = total;
}

void Java_com_sun_cldc_io_palm_http_Inet_closeHTTPsocket()
{
        Handle sockH = (Handle)popStack();
        Err err;

        lastErr = 0;

        oneLess;
        err = INetLibSockClose(AppINetRefnum, sockH);
        if (err) {
                lastErr = err;
                raiseException("com/sun/cldc/io/palm/http/InetException");
				return;
        }
}

void Java_com_sun_cldc_io_palm_http_Inet_getlasterror()
{
        pushStack(lastErr);
}

void FinalizeINetLib()
{
        lastErr = 0;

        if (AppINetRefnum) {
                lastErr = INetLibClose(AppINetRefnum, AppInetH);
				AppInetH = 0;
				AppINetRefnum = 0;
        }
}












